#!/usr/bin/perl
# PROGRAM	: ipactop 0.4.0 (cgi)
# PURPOSE	: prints sorted list of top ipac users (traffic accounting)
# AUTHOR	: Samuel Behan <samkob@gmail.com> (c) 2000-2002
# HOMEPAGE	: http://devel.dob.sk/ipactop
# REQUIRES	: GGI.pm, perl
# LICENSE	: GNU GPL v2 or later, NO WARRANTY VOID (see file LICENSE)
################################################################################
$AUTHOR		= 'Sam';
$AUTHOR_EMAIL	= 'samkob@gmail.com';
$SCRIPT		= 'ipactop';
$VERSION	= '0.4.0';

#CONFIG
#traffic max
$CONFIG_LMAX	= "16";		#kbit/sec (teoretical)
#ipacsum binary
$CONFIG_IPACSUM	= "/usr/local/sbin/ipacsum";

##
# Ipactop script
$|= 1;
use Socket;
use CGI::Pretty;
use POSIX qw( strftime );

#create a query
$q	= new CGI;

#print headers and html start
print $q->header(-type		=> "text/html",
		 -expire 	=> "-1d",
		 -charset	=> "iso-8859-1");
#get params
$script_url	= $q->url(-absolute	=> 1);
$view_mode	= $q->param("time_mode")		|| "0";
$view_frame	= $q->param("time_frame")		|| "today";
$view_last	= $q->param("time_last")		|| "5m";
$view_start	= $q->param("time_start")		|| "epoch";
$view_end	= $q->param("time_end")			|| "now";
$view_quant	= ($q->param("single_units") eq "1")	|| 0;
$view_noresolv	= ($q->param("resolve_off") eq "1")	|| 0;
$view_xml	= ($q->param("xml") eq "1")		|| 0;

#print header
if(!$view_xml) {
print <<EOF
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
    <TITLE>Traffic Top</TITLE>
    <META HTTP-EQUIV="Refresh" CONTENT="120">
    <META HTTP-EQUIV="Cache-Control" content="no-cache">
    <META HTTP-EQUIV="Pragma" CONTENT="no-cache">
    <META HTTP-EQUIV="Expires" CONTENT="Fri, 01 Jan 2000 00:00:01 GMT">
</HEAD>
<BODY bgcolor="#eeeee">
EOF
; }
else {
print <<EOF
<?xml version="1.0" encoding="iso-8859-1"?>
<ipactop>
	<about hidden="1">
		<name>$SCRIPT</name>
		<version>$VERSION</version>
		<author>
			<name>$AUTHOR</name>
			<email>$AUTHOR_EMAIL</email>
		</author>
		<traffic_max>$CONFIG_LMAX</traffic_max>
	</about>
EOF
;
}

#setup time options
$time_start	= "";
$time_end	= "";
$time_desc	= "";
$traff_sec	= 0;
@time_last_vals	= ("5m", "10m", "15m", "30m", 
	"1h", "2h", "3h", "4h", "5h", "6h", "7h", "8h", "9h", "10h", "11h", "12h",
	"13h", "14h", "15h", "16h", "18h", "19h", "20h", "21h", "22h", "23h", "24h",
	"1D", "2D", "3D", "4D", "5D", "6D", "7D",
	"1W", "2W", "3W", "4W",
	"1M", "2M", "3M", "4M", "5M", "6M");
%time_frame_vals	= (
	"today"		=> "Today",
	"yesterday"	=> "Yesterday",
	"this_week"	=> "This Week",
	"this_month"	=> "This Month",
	"this_year"	=> "This Year");
if($view_mode eq "2")
{	#absolute time
	$selected_abs	= " CHECKED";
	#not supported (for now), too many input validations required
}
elsif($view_mode eq "1")
{	#frame based mode
	$selected_frame	= " CHECKED";
	if($view_frame eq "today")		#statistics for today
	{	my($hour)	= strftime("%H", localtime(time())) + 1;
		
		$time_start	= $hour."h";	
		$traff_sec	= $hour * 3600;	}
	elsif($view_frame eq "yesterday")	#statistics fro yesterday
	{	my($hour)	= strftime("%H", localtime(time())) + 1;

		$time_start	= ($hour + 24)."h";
		$time_end	= $hour."h";	
		$traff_sec	= $hour * 3600;	}
	elsif($view_frame eq "this_week")
	{	my($hour)	= strftime("%H", localtime(time())) + 1;
		my($day)	= strftime("%w", localtime(time())) - 1;
		
		$day		= 6 if($day < 0);	#sunday patch
		$time_start	= ($hour + $day*24)."h";	
		$traff_sec	= ($hour + $day * 24) * 3600;	}
	elsif($view_frame eq "this_month")
	{	my($hour)	= strftime("%H", localtime(time())) + 1;
		my($day)	= strftime("%d", localtime(time())) - 1;

		$time_start	= ($hour + $day*24)."h";	
		$traff_sec	= ($hour + $day * 24) * 3600;	}
	elsif($view_frame eq "this_year")
	{	my($day)	= strftime("%j", localtime(time()));

		$time_start	= $day."d";	
		$traff_sec	= $day * 24 * 3600;	}
	$time_desc		= "for $time_frame_vals{$view_frame}" 
			if($time_start);
}
else	#default
{	#last n time
	my(%secs)	= (
		"s"	=> 1,
		"m",	=> 60,
		"h",	=> 3600);

	$view_mode	= 0;
	$selected_last	= " CHECKED";
	$view_last	= "5m"	if($view_last	!~ /^(\d+)(s|m|h|D|W|M|Y)$/o);
	$time_start	= $view_last;
	$time_desc	= "for last $view_last";
	$traff_sec	= $view_last;
	$traff_sec	=~ s/(s|m|h|D|W|M|Y)/\* $secs{$1}/o;
	$traff_sec	= eval($traff_sec);
}

#assemble arguments
$time_opts	= "";
$time_opts	= "-s '$time_start'" 		if($time_start);
$time_opts	= "$time_opts -e '$time_end'"	if($time_end);
$time_opts	= "-s '5m'"			if(!$time_opts);
$time_desc	= "for last 5m"			if(!$time_desc);

#execute ipacsum
@output = `$CONFIG_IPACSUM --fixed-quantity '' $time_opts 2>/dev/null`;
#@output	= `cat ipac.out 2>/dev/null`;
if(!defined(@output))
{
if(!$view_xml) {
print <<EOF
	<FONT size="5" color="red">
		<CENTER><B> Nie je mozne citat databazu !!! </B></CENTER>
	</FONT>
</BODY>
</HTML>
EOF
; }
else {
print <<EOF
	<error>
		<code>1<code>
		<descrition>Database not avaible</description>
	</error>
</ipactop>
EOF
;
}
	exit;
};

#sum bytes
foreach $out (@output)
{
  if   ($out =~ /^\s+(\S+?)(_.+?)?\s+IN\s+\:\s+(\d+)/o)
  {	$ids{$1}	+= int($3);
	$ids_i{$1}	+= int($3);
  }
  elsif($out =~ /^\s+(\S+?)(_.+?)?\s+OUT\s+\:\s+(\d+)/o)
  {	$ids{$1}	+= int($3);
	$ids_o{$1}	+= int($3);
  }
}
undef $out;

#sort hosts
$maxbytes	= 0;
$maxkey		= "";
@ids_keys	= keys(%ids);

#fixme: should sort until
for($i = 0; $i <= ($#ids_keys * 2); $i++)
{	#travel whole arrray
	foreach $key (@ids_keys)
	{	if($ids{$key} > $maxbytes)
		{	$maxbytes	= $ids{$key};
			$maxkey		= $key;	}	}
	if($maxbytes)
	{	push(@sorted, $maxkey);			#push to sorted
		$ids{$maxkey}	= -$ids{$maxkey};	#inverse
		$maxbytes	= 0;	}		#reset
}
#exit;

#print out table
if(!$view_xml) {
print <<EOF
	<TABLE align="center" border="1" cellspacing="0" cellpadding="2">
	<TH colspan="5"> 
		<FONT size="4"> Top traffic $time_desc </FONT>
	</TH>
	<TR>
	  <TD align="center"><B>Num</B></TD>
	  <TD align="center" width="300"><B>Ident</B></TD>
	  <TD align="center"><B>Size (bytes)</B></TD>
	  <TD align="center"><B>Avg (kbits/sec)</B></TD>
	</TR>
EOF
; }
else {
print <<EOF
	<top>
		<time>
			<mode>$view_mode</mode>
			<last>$view_last</last>
			<frame>$view_frame</frame>
		</time>
	<table>
EOF
;
}
	#quantity conversion
	%q2q	= (
		"B"	=>	"K",
		"K"	=>	"M",
		"M"	=>	"G",
		"G"	=>	"T");
	#printout table
	$num	= 0;
	foreach $key (@sorted)
	{	$num++;
		$host	=  ((!$view_noresolv) && ($key =~ /^\d/o)) ? 
				gethostbyaddr(inet_aton($key), AF_INET) || $key :
				$key;
		$ids{$key}	= -$ids{$key};
		$avg		= $ids{$key} / $traff_sec if ($traff_sec);
		$quant		= "B";
		#change quantity (K, M...)
		while(!$view_quant && defined($q2q{$quant}) && $ids{$key} / 1000 > 10)
		{	$ids{$key}	= $ids{$key} / 1000;
			$quant		= $q2q{$quant};	}
		$count		= sprintf("%.2f", $ids{$key});
		$avg_print	= "";
		if($traff_sec)
		{	$avg		= $avg	/ 1000;
			$avg_percent	= sprintf("%2.2f", $avg / $CONFIG_LMAX * 100);
			$avg		= sprintf("%.2f", $avg);
			$avg_print	= "$avg [$avg_percent %]";
		}
if(!$view_xml) {
print <<EOF
	<TR>
		<TD align="center"><B>$num</B></TD>
		<TD><B>$host</B></TD>
		<TD align="center">$count $quant</TD>
		<TD align="center">$avg_print</TD>
	</TR>
EOF
; }
else {
print <<EOF
	<host>
		<num>$num</num>
		<name>$host</name>
		<real_name>$key</real_name>
		<traffic>
			<sum>$ids{$key}</sum>
			<in>$ids_i{$key}</in>
			<out>$ids_o{$key}</out>
		</traffic>
	</host>
EOF
;
}
	}

#html end
if(!$view_xml) {
print <<EOF
	</TABLE>
	<FORM action="$script_url" method="GET">
	<TABLE align="center" border="0" cellspacing="0" cellpadding="2">
	<TH colspan="3">
		<B> View setup </B>
	</TH>
	<TR>
		<TD><INPUT type="radio" name="time_mode" value="0"$selected_last></TD>
		<TD>Show last</TD>
		<TD>
			<SELECT name="time_last" width="50">
EOF
; }
else {
print <<EOF
	</table>
	</top>
	<args hidden="1">
	<time>
		<last>
			<mode>0</mode>
			<name>time_last</name>
EOF
;
}
	foreach $val (@time_last_vals)
	{
		$selected	= "";
if(!$view_xml) {
		$selected	= " SELECTED" if($view_last eq $val);
print <<EOF
			<OPTION value="$val"$selected>$val</OPTION>
EOF
; }
else {
		$selected	= " selected=\"1\"" if($view_last eq $val);
print <<EOF
			<option$selected>$val</option>
EOF
; }
	}
	$selected	= "";
	$selected	= " CHECKED" if($view_quant);
if(!$view_xml) {
print <<EOF
			</SELECT>
		</TD>
	</TR>
	<TR>
		<TD><INPUT type="radio" name="time_mode" value="1"$selected_frame></TD>
		<TD>Show frame</TD>
		<TD>
			<SELECT name="time_frame" width="50">
EOF
; }
else {
print <<EOF
		</last>
		<frame>
			<mode>1</mode>
			<name>time_frame</name>
EOF
;
}
	foreach $val (keys(%time_frame_vals))
	{
		$selected	= "";
if(!$view_xml) {
		$selected	= " SELECTED" if($view_frame eq $val);
print <<EOF
			<OPTION value="$val"$selected>$time_frame_vals{$val}</OPTION>
EOF
; }
else {
		$selected	= " selected=\"1\"" if($view_frame eq $val);
print <<EOF
			<option$selected>$val</option>
EOF
;
}
	}
	$selected	= "";
	$selected	= " CHECKED" if($view_quant);
if(!$view_xml) {
print <<EOF			
			</SELECT>
		</TD>
	</TR>
	<TR>
		<TD>
			<INPUT name="single_units" type="checkbox" value="1"$selected>
		</TD>
		<TD colspan="2">Bytes only</TD>
	<TR>
EOF
;
	$selected	= "";
	$selected	= " CHECKED" if($view_noresolv);
print <<EOF
	<TR>
		<TD>
			<INPUT name="resolve_off" type="checkbox" value="1"$selected>
		</TD>
		<TD colspan="2">No resolv</TD>
	<TR>
	<TR>
		<TD>
			<INPUT name="xml" type="checkbox" value="1">
		</TD>
		<TD colspan="2">XML view (no refresh)</TD>
	<TR>
	<TR>
		<TD colspan="3" align="center">
			<INPUT type="submit">
		</TD>
	<TR>
	</TABLE>
	</FORM>

	<BR/><BR/>
	<CENTER>
        <HR width="60%">
	<FONT size="2" color="red">\u$SCRIPT $VERSION by <A href="$AUTHOR_EMAIL">$AUTHOR</A></FONT>
	</CENTER>
</BODY>
</HTML>
EOF
; }
else {
print <<EOF
		</frame>
	</time>
	</args>
</ipactop>
EOF
; }

__END__

